@@ -4,14 +4,19 @@
  *
  * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
  *
- * $Id: auth_pam.h,v 1.5 2001/01/21 12:30:50 chris Exp $
+ * $Id: auth_pam.h,v 1.6 2001/04/23 23:45:40 chris Exp $
  *
  */
 
 #ifndef __AUTH_PAM_H_ /* include guard */
 #define __AUTH_PAM_H_
 
+#ifdef HAVE_CONFIG_H
+#include "configuration.h"
+#endif /* HAVE_CONFIG_H */
+
 #ifdef AUTH_PAM
+
 #include "authswitch.h"
 
 /* config directive auth-pam-mailspool-dir */
