@@ -9,7 +9,11 @@
  *
  */
 
-static const char rcsid[] = "$Id: cfgdirectives.c,v 1.1 2001/03/21 19:43:34 chris Exp $";
+static const char rcsid[] = "$Id: cfgdirectives.c,v 1.4 2001/04/23 23:45:40 chris Exp $";
+
+#ifdef HAVE_CONFIG_H
+#include "configuration.h"
+#endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
 
@@ -19,19 +23,20 @@ char *cfgdirectives[] = {
     "max-children",
     "append-domain",
     "timeout-seconds",
+    "mailbox",
 
 #ifdef AUTH_PAM
     /* auth-pam options */
     "auth-pam-enable",
     "auth-pam-facility",
-    "auth-pam-mailspool-dir",
+    "auth-pam-mailbox",
     "auth-pam-mail-group",
 #endif
 
 #ifdef AUTH_PASSWD
     /* auth-passwd options */
     "auth-passwd-enable",
-    "auth-passwd-mailspool-dir",
+    "auth-passwd-mailbox",
     "auth-passwd-mail-group",
 #endif
 
@@ -43,8 +48,16 @@ char *cfgdirectives[] = {
     "auth-mysql-database",
     "auth-mysql-hostname",
     "auth-mysql-mail-group",
-#endif
+#endif /* AUTH_MYSQL */
 
+#ifdef AUTH_OTHER
+    "auth-other-enable",
+    "auth-other-program",
+    "auth-other-user",
+    "auth-other-group",
+    "auth-other-timeout",
+#endif /* AUTH_OTHER */
+    
     /* final entry must be NULL */
     NULL};
 
