@@ -6,7 +6,11 @@
  *
  */
 
-static const char rcsid[] = "$Id: connection.c,v 1.20 2001/03/14 12:43:29 chris Exp $";
+static const char rcsid[] = "$Id: connection.c,v 1.23 2001/04/23 23:45:40 chris Exp $";
+
+#ifdef HAVE_CONFIG_H
+#include "configuration.h"
+#endif /* HAVE_CONFIG_H */
 
 #include <errno.h>
 #include <fcntl.h>
@@ -130,7 +134,7 @@ void connection_delete(connection c) {
     close(c->s);
 
     if (c->a) authcontext_delete(c->a);
-    if (c->m) mailspool_delete(c->m);
+    if (c->m) (c->m)->delete(c->m);
 
     if (c->domain)    free(c->domain);
     if (c->idstr)     free(c->idstr);
@@ -149,7 +153,7 @@ ssize_t connection_read(connection c) {
     ssize_t n;
     if (!c) return -1;
     if (c->p == c->buffer + c->bufferlen) {
-        print_log(LOG_ERR, "connection_read: client %s: over-long line", c->idstr);
+        print_log(LOG_ERR, _("connection_read: client %s: over-long line"), c->idstr);
         errno = ENOBUFS;
         return -1;
     }
@@ -217,10 +221,10 @@ pop3command connection_parsecommand(connection c) {
     if (verbose) {
         char *s;
         int i, l;
-        l = sizeof("connection_parsecommand: client : received `'") + strlen(c->idstr);
+        l = strlen(_("connection_parsecommand: client %s: received `")) + 2 + strlen(c->idstr);
         for (i = 0; i < pc->toks->toks->n_used; ++i) l += strlen((char*)(pc->toks->toks->ary[i].v)) + 6;
         s = (char*)malloc(l);
-        sprintf(s, "connection_parsecommand: client %s: received `", c->idstr);
+        sprintf(s, _("connection_parsecommand: client %s: received `"), c->idstr);
         for (i = 0; i < pc->toks->toks->n_used; ++i) {
             if (i == 0 || pc->cmd != PASS) strcat(s, (char*)(pc->toks->toks->ary[i].v));
             else strcat(s, "[...]");
@@ -286,7 +290,7 @@ int connection_sendresponse(connection c, const int success, const char *s) {
     m = xwrite(c->s, x, l = strlen(x));
     free(x);
     if (verbose)
-        print_log(LOG_DEBUG, "connection_sendresponse: client %s: sent `%s %s'", c->idstr, success? "+OK" : "-ERR", s);
+        print_log(LOG_DEBUG, _("connection_sendresponse: client %s: sent `%s %s'"), c->idstr, success? "+OK" : "-ERR", s);
     return (m == l);
 }
 
