@@ -6,7 +6,11 @@
  *
  */
 
-static const char rcsid[] = "$Id: listener.c,v 1.1 2001/04/03 16:10:22 chris Exp $";
+static const char rcsid[] = "$Id: listener.c,v 1.3 2001/04/21 03:49:24 chris Exp $";
+
+#ifdef HAVE_CONFIG_H
+#include "configuration.h"
+#endif /* HAVE_CONFIG_H */
 
 #include <fcntl.h>
 #include <netdb.h>
@@ -63,8 +67,8 @@ listener listener_new(const struct sockaddr_in *addr, const char *domain) {
     if (!domain) {
         he = gethostbyaddr((char *)&(addr->sin_addr), sizeof(addr->sin_addr), AF_INET);
         if (!he) {
-            print_log(LOG_WARNING, "listener_new: gethostbyaddr(%s): cannot resolve name", inet_ntoa(addr->sin_addr));
-            print_log(LOG_WARNING, "listener_new: %s: no domain suffix can be appended for this address", inet_ntoa(addr->sin_addr));
+            print_log(LOG_WARNING, _("listener_new: gethostbyaddr(%s): cannot resolve name"), inet_ntoa(addr->sin_addr));
+            print_log(LOG_WARNING, _("listener_new: %s: no domain suffix can be appended for this address"), inet_ntoa(addr->sin_addr));
         } else {
             /* We need to find out an appropriate domain suffix for the address.
              * FIXME we just take the first address with a "." in it, and use the
@@ -85,7 +89,7 @@ listener listener_new(const struct sockaddr_in *addr, const char *domain) {
                 }
 
             if (!L->domain)
-                print_log(LOG_WARNING, "listener_new: %s: no suitable domain suffix found for this address", inet_ntoa(addr->sin_addr));
+                print_log(LOG_WARNING, _("listener_new: %s: no suitable domain suffix found for this address"), inet_ntoa(addr->sin_addr));
         }
     } else L->domain = strdup(domain);
 
@@ -94,10 +98,10 @@ listener listener_new(const struct sockaddr_in *addr, const char *domain) {
         struct utsname u;
         if (uname(&u) == -1) {
             print_log(LOG_WARNING, "listener_new: uname: %m");
-            print_log(LOG_WARNING, "listener_new: %s: using domain suffix `x.invalid'", inet_ntoa(addr->sin_addr));
+            print_log(LOG_WARNING, _("listener_new: %s: using domain suffix `x.invalid'"), inet_ntoa(addr->sin_addr));
             L->domain = strdup("x.invalid");
         } else {
-            print_log(LOG_WARNING, "listener_new: %s: using fallback domain suffix %s", inet_ntoa(addr->sin_addr), u.nodename);
+            print_log(LOG_WARNING, _("listener_new: %s: using fallback domain suffix `%s'"), inet_ntoa(addr->sin_addr), u.nodename);
             L->domain = strdup(u.nodename);
         }
     }
