@@ -6,7 +6,11 @@
  *
  */
 
-static const char rcsid[] = "$Id: signals.c,v 1.3 2001/04/03 20:19:13 chris Exp $";
+static const char rcsid[] = "$Id: signals.c,v 1.7 2001/04/23 23:45:40 chris Exp $";
+
+#ifdef HAVE_CONFIG_H
+#include "configuration.h"
+#endif /* HAVE_CONFIG_H */
 
 #include <signal.h>
 #include <syslog.h>
@@ -14,6 +18,7 @@ static const char rcsid[] = "$Id: signals.c,v 1.3 2001/04/03 20:19:13 chris Exp
 #include <sys/wait.h>
 
 #include "connection.h"
+#include "pidfile.h"
 #include "signals.h"
 #include "util.h"
 
@@ -31,13 +36,13 @@ void appalling_backtrace_hack() {
 
     n = backtrace(func_addr, BT_LEVELS);
 
-    print_log(LOG_ERR, "appalling_backtrace_hack: stack trace of program begins");
+    print_log(LOG_ERR, _("appalling_backtrace_hack: stack trace of program begins"));
     
     for (i = 0; i < n; ++i)
         print_log(LOG_ERR, "appalling_backtrace_hack:    [%d]: %p", i, func_addr[i]);
 
-    print_log(LOG_ERR, "appalling_backtrace_hack: stack trace of program ends");
-    print_log(LOG_ERR, "appalling_backtrace_hack: use addr2line(1) to resolve the addresses");
+    print_log(LOG_ERR, _("appalling_backtrace_hack: stack trace of program ends"));
+    print_log(LOG_ERR, _("appalling_backtrace_hack: use addr2line(1) to resolve the addresses"));
 }
 #endif
 
@@ -101,7 +106,7 @@ void set_signals() {
 extern int foad;                            /* in main.c */
 
 void terminate_signal_handler(const int i) {
-    foad = 1;
+    foad = i;
 }
 
 /* die_signal_handler:
@@ -114,14 +119,20 @@ void terminate_signal_handler(const int i) {
  */
 extern connection this_child_connection;    /* in main.c */
 
+extern char * pidfile;    /* in main.c */
+extern int post_fork;    /* in main.c */
+
 void die_signal_handler(const int i) {
     struct sigaction sa;
 /*    print_log(LOG_ERR, "quit: %s", sys_siglist[i]); */
-    print_log(LOG_ERR, "quit: signal %d", i); /* Some systems do not have sys_siglist. */
+    print_log(LOG_ERR, _("quit: signal %d"), i); /* Some systems do not have sys_siglist. */
 #ifdef APPALLING_BACKTRACE_HACK
     appalling_backtrace_hack();
 #endif /* APPALLING_BACKTRACE_HACK */
     if (this_child_connection) connection_delete(this_child_connection);
+    if (!post_fork)
+        if (pidfile)
+            remove_pid_file(pidfile);
     memset(&sa, 0, sizeof(sa));
     sa.sa_handler = SIG_DFL;
     sigaction(i, &sa, NULL);
@@ -131,13 +142,27 @@ void die_signal_handler(const int i) {
 /* child_signal_handler:
  * Signal handler to deal with SIGCHLD.
  */
-extern int num_running_children;            /* in main.c */
+extern int num_running_children; /* in main.c */
+
+#ifdef AUTH_OTHER
+extern pid_t authchild_pid; /* in auth_other.c */
+extern int authchild_wr, authchild_rd;
+#endif /* AUTH_OTHER */
 
 void child_signal_handler(const int i) {
+    pid_t pid;
     int status;
     
-    while (waitpid(-1, &status, WNOHANG) > 0)
-        --num_running_children;
+    while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
+#ifdef AUTH_OTHER
+        if (pid == authchild_pid) {
+            authchild_pid = 0;
+            print_log(LOG_WARNING, _("child_signal_handler: authentication child %d terminated; exit status was %d"), (int)pid, status);
+            close(authchild_wr);
+            close(authchild_rd);
+        } else
+#endif /* AUTH_OTHER */
+            --num_running_children;
 }
 
 /* restart_signal_handler:
@@ -146,7 +171,10 @@ void child_signal_handler(const int i) {
 extern int restart, post_fork;              /* in main.c */
 
 void restart_signal_handler(const int i) {
-    if (!post_fork) foad = restart = 1;
+    if (!post_fork) {
+        foad = i;
+        restart = 1;
+    }
 }
 
 
