@@ -6,7 +6,11 @@
  *
  */
 
-static const char rcsid[] = "$Id: tokenise.c,v 1.3 2001/01/21 12:30:50 chris Exp $";
+static const char rcsid[] = "$Id: tokenise.c,v 1.5 2001/04/21 03:49:24 chris Exp $";
+
+#ifdef HAVE_CONFIG_H
+#include "configuration.h"
+#endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
 #include <string.h>
